% The following program detects the fraktion peak onset latency on
% data which was sent from Brain Vision Analyzer 2.0.2 (or higher)
% to Matlab through the Matlab transformation. 

% It is intended to be used on time-domain averaged nodes after
% peak detection transformation marked the peak(s). The data is
% intended to be exported with the following options:
% First window of the User Interface
% -'Calculate Data on Creation of Node'
% -'Code Executed on Creation of Node' field should contain either
%  all lines of this file or should execute this file
% Second window of the User Interface
% -'Export Marker checked
% -'Delete Original Markers after Export' checked
% -all other checkboxes are not checked
% Third window of the User Interface
% -'Export All Channels'

% As an output, the program places a new Peak marker to the first 
% data point, where the user defined fraction of the peak amplitude
% is reached. The detection is made backwards in order to mark the
% onset latency

% Author: Dr. Roland Csuhaj
% 31/10/2012
% Brain Products GmbH
% support@brainproducts.com

% This is free example program, not a commercial product, not 
% part of the Brain Vision Analyzer. The user accepts it 'as is'.
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY.

Threshold = 50; %change the threshold here (percentage, between 1 and 99)

%%%%%%%%%%%%
% Please note, the New Markers has to be saved in the variable called
% 'NewMarkers'. Do not alter this name, otherwise Analyzer will not find it
%%%%%%%%%%%%

NewMarkers = Markers; %copy the old markers in order to keep them. It is not necessary, if the 'Delete Original Markers after Export' checkbox is not activated in Analyzer
for i = 1 : size(Markers,2)	%check all markers one-by-one
    if   strcmp(Markers(1,i).Type,('Peak'))	%if a Peak marker is found, start the threshold detection
        
		%%%%%%%%%%%%%
		% Please note, the properties of Markers follow the common practice of C#,
		% where the index starts with 0, while in Matlab the index starts with 1. 
		% Therefore the Markers.Channel and the Markers.Postion properties have to
		% be increased by one, if you want to refer to the correct cell of the
		% EEGData matrix.
		% Similarly, the indexes of the EEGData cell have to be decreased by one,
		% if you would like to create a new marker for Analyzer.
		%%%%%%%%%%%%%
		
		Pos =  Markers(1,i).Position+1;				%stores the position of the peak marker according to the EEGData indexes
		PCh = Markers(1,i).Channel+1;					%stores the channel number of the peak marker according to the EEGData indexes
        PValue = abs(EEGData(Pos, PCh)); 			%uses the absolute value of the peak in order to handle positive and negative peaks in together		
		Pos = Pos - 1; 								%the detection goes backwards
        while Pos > 0 									%this loop detects the threshold
            CurrentValue = abs(EEGData(Pos, PCh));		%absolute value is used again
            if CurrentValue <= PValue*Threshold*0.01 %if the value is equal or below the threshold
                x = size(NewMarkers,2) +1;				%add a new marker
                NewMarkers(1,x) = Markers(1,i);			%which is the same as the Peak marker, but
                NewMarkers(1,x).Description = strcat(Markers(1,i).Description, '_', num2str(Threshold)); % with a new description and 
                NewMarkers(1,x).Position = Pos - 1; 	%with new position. Please note, the position is decreased by one because of the c#-matlab differences
                break
            end
            Pos = Pos - 1;							%goes to the previous data point
        end
    end
end